/***********************************************************************
tcping.exe -- A tcp probe utility
Copyright (C) 2005 Eli Fulkerson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

----------------------------------------------------------------------

Other license terms may be negotiable.  Contact the author if you would
like a copy that is licensed differently.

Contact information (as well as this program) lives at http://www.elifulkerson.com

----------------------------------------------------------------------

This application includes public domain code from the Winsock Programmer's FAQ:
  http://www.tangentsoft.net/wskfaq/
... and a big thank you to the maintainers and contributers therein.

***********************************************************************/

#include <winsock2.h>

#include <stdlib.h>
#include <iostream>

using namespace std;


//// Prototypes ////////////////////////////////////////////////////////

extern int DoWinsock(char* pcHost, int nPort, int times_to_ping, float ping_interval, int include_timestamp, int beep_mode, int ping_timeout, int relookup_interval, int auto_exit_on_success, int old_style_time, int force_send_byte, int include_url, int use_http, char* docptr, int post);


//// Constants /////////////////////////////////////////////////////////

// Default port to connect to on the server
const int kDefaultServerPort = 80;




//// main //////////////////////////////////////////////////////////////

int main(int argc, char* argv[])
{
	
    // Do we have enough command line arguments?
    if (argc < 2) {
		//cout << "--------------------------------------------------------------" << endl;
		//cout << "tcping.exe by Eli Fulkerson " << endl;
		//cout << "Please see http://www.elifulkerson.com/projects/ for updates. " << endl;
		//cout << "--------------------------------------------------------------" << endl;
        cout << "Usage: " << argv[0] << " [-t] [-d] [-i interval] [-n times] server-address " <<
                "[server-port]" << endl << endl;
        cout << "\t -t   : ping continuously until stopped via control-c" << endl;
        cout << "\t -n 5 : for instance, send 5 pings" << endl;
	cout << "\t -i 5 : for instance, ping every 5 seconds" << endl;
	cout << "\t -w 100 : for instance, wait 100 milliseconds for a response" << endl;   
	cout << "\t -d   : include date and time on each line" << endl;
	cout << "\t -b 1 : enable beeps (1 for on-down, 2 for on-up," << endl;
	cout << "\t                      3 for on-change, 4 for always)" << endl;
	cout << "\t -r 5 : for instance, relookup the hostname every 5 pings" << endl;
	cout << "\t -s   : automatically exit on a successful ping"<<endl;                  //[Modification 14 Apr 2011 by Michael Bray, mbray@presidio.com]
	cout << "\t -v   : print version and exit" << endl;
	cout << "\t -d   : include date and time on each line" << endl;
	cout << endl << "HTTP Options:" << endl;
	cout << "\t -h   : HTTP mode (use url without http:// for server-address)" << endl;
	cout << "\t -u   : include target URL on each line" << endl;
	cout << "\t -p   : use POST rather than GET (to avoid caching)" << endl;
	cout << endl << "Debug Options:" << endl;
	cout << "\t -o   : use alternate timing function (less precise)" << endl;
	cout << "\t -f   : force tcping to send at least one byte" << endl;

	cout << endl << "\tIf you don't pass server-port, it defaults to " << kDefaultServerPort << "." << endl;
	    

        return 1;
    }

	int times_to_ping = 4;
	int offset = 0;  // because I don't feel like writing a whole command line parsing thing, I just want to accept an optional -t
	float ping_interval = 1;
	int include_timestamp = 0;
	int beep_mode = 0;  // 0 is off, 1 is down, 2 is up, 3 is on change, 4 is constantly
	int ping_timeout = 2000;
    int relookup_interval = -1;
	int auto_exit_on_success = 0;
    int old_style_time = 0; // use the inaccurate time
    int force_send_byte = 0;
    
	int include_url = 0;
	int use_http = 0;
	int use_post = 0;

	
	// for http mode
    const char *serverptr;
    char *docptr;
    char server[2048];
    char document[2048];	
	    
    
    for (int x=0; x < argc; x++) {
        // ping continuously
	if (!strcmp(argv[x], "-t")) {
            times_to_ping = -1;
            offset = x;
            cout << endl << "** Pinging continuously.  Press control-c to stop **" << endl;
        }
        
	// Number of times to ping
        if (!strcmp(argv[x], "-n")) {
            times_to_ping = atoi(argv[x+1]);
            offset = x+1;
        }

    // exit on first successful ping
	if (!strcmp(argv[x], "-s")) {
		auto_exit_on_success = 1;
		offset = x;
	}
	
	// http mode
	 if (!strcmp(argv[x], "-h")) {
            use_http = 1;
            offset = x;
        }
        
	// http mode
	 if (!strcmp(argv[x], "-p")) {
            use_post = 1;
            offset = x;
        }        
	
	
	// include url per line
	 if (!strcmp(argv[x], "-u")) {
            include_url = 1;
            offset = x;
        }
	
		
	// use old timer
	if (!strcmp(argv[x], "-o")) {
		old_style_time = 1;
		offset = x;
	}
	
	// force send a byte
	if (!strcmp(argv[x], "-f")) {
		force_send_byte = 1;
		offset = x;
	}
        
        
	// interval between pings
	if (!strcmp(argv[x], "-i")) {
            ping_interval = atof(argv[x+1]);
            offset = x+1;
        }
	
	// wait for response
	if (!strcmp(argv[x], "-w")) {
            ping_timeout = atoi(argv[x+1]);
            offset = x+1;
        }
	
	// optional datetimestamp output
	 if (!strcmp(argv[x], "-d")) {
            include_timestamp = 1;
            offset = x;
        }
	
	// optional hostname re-lookup
	 if (!strcmp(argv[x], "-r")) {
            relookup_interval = atoi(argv[x+1]);
            offset = x+1;
        }
	
	// optional beepage
	if (!strcmp (argv[x], "-b")) {
		beep_mode = atoi(argv[x+1]);
		offset = x+1;
		switch (beep_mode) {
			case 0:
				break;
			case 1:
				cout << endl << "** Beeping on \"down\" - (two beeps) **" << endl;
				break;
			case 2:
				cout << endl << "** Beeping on \"up\"  - (one beep) **" << endl;
				break;
			case 3:
				cout << endl << "** Beeping on \"change\" - (one beep up, two beeps down) **" << endl;
				break;
			case 4:
				cout << endl << "** Beeping constantly - (one beep up, two beeps down) **" << endl;
				break;
		}
		
	}
	
	// dump version and quit
	if (!strcmp(argv[x], "-v")) {
		cout << "tcping.exe 0.14 Jul 20 2012" << endl;
		cout << "compiled: " << __DATE__ << " " << __TIME__ << endl;
		cout << endl;
		cout << "tcping.exe by Eli Fulkerson " << endl;
		cout << "Please see http://www.elifulkerson.com/projects/ for updates. " << endl;
		cout << endl;
		cout << "-s option contributed 14 Apr 2011 by Michael Bray, mbray@presidio.com" << endl;
		return 1;
	}
    }

    // Get host and (optionally) port from the command line
    char* pcHost = argv[1 + offset];
    int nPort = kDefaultServerPort;
    if (argc >= 3 + offset) {
        nPort = atoi(argv[2 + offset]);
    }

    // Do a little sanity checking because we're anal.
    int nNumArgsIgnored = (argc - 3 - offset);
    if (nNumArgsIgnored > 0) {
        cout << nNumArgsIgnored << " extra argument" <<
                (nNumArgsIgnored == 1 ? "" : "s") << 
                " ignored.  FYI." << endl;
    }
    
    if (use_http == 1) {
		serverptr = strchr(pcHost, ':');
    	if (serverptr != NULL) {
			++serverptr;
			++serverptr;
			++serverptr;
    	} else {
			serverptr = pcHost;
    	}
    	//cout << serverptr << endl;
    
    	docptr = strchr(serverptr, '/');
    	if (docptr != NULL) {
			*docptr = '\0';
			++docptr;

			strcpy( server, static_cast<const char*>(serverptr) );
			strcpy( document, static_cast<const char*>(docptr) );
    	} else {
			//cout << "found nothing, no document";
			//strcpy( document, static_cast<const char*>('/') );
			strcpy( server, static_cast<const char*>(serverptr) );
			//document[0] = '/';
			//document[1] = '\0';
			document[0] = '\0';
    	}
    
 		   
    	cout << "** Requesting \"" << document << "\" from " << server << ":" << endl;
    	cout << "(for various reasons, kbit/s is an approximation)" << endl;
    }
    

    // Start Winsock up
    WSAData wsaData;
	int nCode;
    if ((nCode = WSAStartup(MAKEWORD(1, 1), &wsaData)) != 0) {
		cout << "WSAStartup() returned error code " << nCode << "." <<
				endl;
        return 255;
    }

    // Call the main example routine.
    int retval = DoWinsock(pcHost, nPort, times_to_ping, ping_interval, include_timestamp, beep_mode, ping_timeout, relookup_interval, auto_exit_on_success, old_style_time, force_send_byte, include_url, use_http, docptr, use_post);

    // Shut Winsock back down and take off.
    WSACleanup();
    return retval;
}

